/* interface.c --- creation of the UI objects  */

/* Copyright (c) E. Lassauge, 2000.
 *
 * Permission to use, copy, modify, and distribute this software and its
 * documentation for any purpose and without fee is hereby granted,
 * provided that the above copyright notice appear in all copies and that
 * both that copyright notice and this permission notice appear in
 * supporting documentation.
 *
 * This file is provided AS IS with no warranties of any kind.  The author
 * shall have no liability with respect to the infringement of copyrights,
 * trade secrets or any patents by this file or any part thereof.  In no
 * event will the author be liable for any lost revenue or profits or
 * other special, indirect and consequential damages.
 *
 * mailto:lassauge@mail.dotcom.fr
 * http://lassauge.free.fr/
 *
 * REVISION HISTORY:
 *
 */

#ifdef HAVE_CONFIG_H
#  include <config.h>
#endif

#include <dirent.h>
#include <sys/stat.h>
#include <unistd.h>

#include <gnome.h>

#include "sms.h"
#include "callbacks.h"
#include "interface.h"
#include "support.h"

static void scan_dir(gchar *dirname);
gint numprovider;

GtkWidget *create_about()
{
    /* The about box */
    const gchar *authors[] = {
	"Eric Lassauge <lassauge@mail.dotcom.fr>",
	NULL
    };
    GtkWidget *about;

    about = gnome_about_new("Gnome SmsSend",
                            VERSION,
			    _("Copyright 2000"), 
                            authors,
                            _("GNOME interface to smssend package"), 
#ifndef DEVEL
			    "gsmssend/gsms-about.png"
#else
			    "../pixmaps/gsms-about.png"
#endif
			    );
    gtk_window_set_modal(GTK_WINDOW(about), TRUE);

    return about;
}

GtkWidget *create_scripts()
{
    /* create a script file selection dialog */

    scripts = gtk_file_selection_new(_("Choose script"));
    gtk_signal_connect (GTK_OBJECT(scripts), "destroy",
		(GtkSignalFunc) gtk_widget_destroy, GTK_OBJECT (scripts));
    gtk_signal_connect(GTK_OBJECT (GTK_FILE_SELECTION (scripts)->ok_button),
		"clicked", (GtkSignalFunc) on_script_ok_pressed, scripts);
    gtk_signal_connect_object(GTK_OBJECT (GTK_FILE_SELECTION
		    (scripts)->cancel_button),
		"clicked", (GtkSignalFunc) gtk_widget_destroy,
		GTK_OBJECT (scripts));
    return scripts;
}

GtkWidget *create_preferences()
{
    /* create the preference dialog box */
    GtkWidget *preferences;
    GtkWidget *dialog_vbox;
    GtkWidget *main_vbox;
    GtkWidget *pref_proxy;
    GtkWidget *proxy_label;
    GtkWidget *use_proxy;
    GtkWidget *proxy_table;
    GtkWidget *proxyhost_label;
    GtkWidget *proxyport_label;
    GtkWidget *proxy_host;
    GtkWidget *proxy_port;
    GtkWidget *proxyuser_label;
    GtkWidget *proxypass_label;
    GtkWidget *proxy_user;
    GtkWidget *proxy_password;
    GtkWidget *config_vbox;
    GtkWidget *config_label;
    GtkWidget *config_hbox1;
    GtkWidget *config_hbox2;
    GtkWidget *debug_label;
    GtkObject *debug_level_adj;
    GtkWidget *debug_level;
    GtkWidget *tmout_label;
    GtkObject *timeout_level_adj;
    GtkWidget *timeout_level;
    GtkWidget *auto_check;
    GtkWidget *keep_pass;
    GtkWidget *dialog_action_area;
    GtkWidget *ok_button;
    GtkWidget *apply_button;
    GtkWidget *cancel_button;

    preferences = gnome_dialog_new(_("Preferences"), NULL);
    gnome_dialog_set_close(GNOME_DIALOG(preferences), TRUE);
    gtk_window_set_policy(GTK_WINDOW(preferences), FALSE, FALSE, FALSE);
    gtk_window_set_wmclass(GTK_WINDOW(preferences), "Smssend", "Smssend");

    dialog_vbox = GNOME_DIALOG(preferences)->vbox;
    gtk_widget_show(dialog_vbox);

    main_vbox = gtk_vbox_new(FALSE, 0);
    gtk_widget_show(main_vbox);
    gtk_box_pack_start(GTK_BOX(dialog_vbox), main_vbox, TRUE, TRUE, 0);

    proxy_label = gtk_frame_new(_("Proxy configuration"));
    gtk_widget_show(proxy_label);
    gtk_box_pack_start(GTK_BOX(main_vbox), proxy_label, TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(proxy_label), 2);
    gtk_frame_set_label_align(GTK_FRAME(proxy_label), 0.02,0.5);

    pref_proxy = gtk_vbox_new(FALSE, 2);
    gtk_widget_set_name(pref_proxy, "pref_proxy");
    gtk_object_set_data(GTK_OBJECT(preferences), "pref_proxy", pref_proxy);
    gtk_widget_show(pref_proxy);
    gtk_container_add(GTK_CONTAINER(proxy_label), pref_proxy);
    gtk_container_set_border_width(GTK_CONTAINER(pref_proxy), 2);

    use_proxy = gtk_check_button_new_with_label(_("Use proxy"));
    gtk_widget_set_name(use_proxy, "use_proxy");
    gtk_object_set_data(GTK_OBJECT(preferences), "use_proxy", use_proxy);
    gtk_widget_show(use_proxy);
    gtk_box_pack_start(GTK_BOX(pref_proxy), use_proxy, FALSE, FALSE, 0);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(use_proxy), UseProxy);

    proxy_table = gtk_table_new(4, 2, FALSE);
    gtk_widget_show(proxy_table);
    gtk_box_pack_end(GTK_BOX(pref_proxy), proxy_table, FALSE, FALSE, 0);
    gtk_table_set_row_spacings(GTK_TABLE(proxy_table), 1);
    gtk_table_set_col_spacings(GTK_TABLE(proxy_table), 2);

    proxyhost_label = gtk_label_new(_("Proxy Host"));
    gtk_widget_show(proxyhost_label);
    gtk_table_attach(GTK_TABLE(proxy_table), proxyhost_label, 0, 1, 0, 1, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_justify(GTK_LABEL(proxyhost_label), GTK_JUSTIFY_RIGHT);

    proxyport_label = gtk_label_new(_("Proxy Port"));
    gtk_widget_show(proxyport_label);
    gtk_table_attach(GTK_TABLE(proxy_table), proxyport_label, 0, 1, 1, 2, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_justify(GTK_LABEL(proxyport_label), GTK_JUSTIFY_RIGHT);

    proxy_host = gtk_entry_new_with_max_length(P_LENGTH-1);
    gtk_widget_set_name(proxy_host, "proxy_host");
    gtk_object_set_data(GTK_OBJECT(preferences), "proxy_host", proxy_host);
    gtk_widget_show(proxy_host);
    gtk_table_attach(GTK_TABLE(proxy_table), proxy_host, 1, 2, 0, 1,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    if (UseProxy)
	gtk_entry_set_text(GTK_ENTRY(proxy_host), ProxyHost);

    proxy_port = gtk_entry_new_with_max_length(4);
    gtk_widget_set_name(proxy_port, "proxy_port");
    gtk_object_set_data(GTK_OBJECT(preferences), "proxy_port", proxy_port);
    gtk_widget_show(proxy_port);
    gtk_table_attach(GTK_TABLE(proxy_table), proxy_port, 1, 2, 1, 2,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    if (UseProxy)
    {
	char port[5];
	g_snprintf(port, 5, "%d", ProxyPort);
	gtk_entry_set_text(GTK_ENTRY(proxy_port), port);
    }

    proxyuser_label = gtk_label_new(_("Proxy User"));
    gtk_widget_show(proxyuser_label);
    gtk_table_attach(GTK_TABLE(proxy_table), proxyuser_label, 0, 1, 2, 3, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_justify(GTK_LABEL(proxyuser_label), GTK_JUSTIFY_RIGHT);

    proxypass_label = gtk_label_new(_("Proxy Password"));
    gtk_widget_show(proxypass_label);
    gtk_table_attach(GTK_TABLE(proxy_table), proxypass_label, 0, 1, 3, 4, (GtkAttachOptions) (0), (GtkAttachOptions) (0), 0, 0);
    gtk_label_set_justify(GTK_LABEL(proxypass_label), GTK_JUSTIFY_RIGHT);

    proxy_user = gtk_entry_new_with_max_length(P_LENGTH-1);
    gtk_widget_set_name(proxy_user, "proxy_user");
    gtk_object_set_data(GTK_OBJECT(preferences), "proxy_user", proxy_user);
    gtk_widget_show(proxy_user);
    gtk_table_attach(GTK_TABLE(proxy_table), proxy_user, 1, 2, 2, 3,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    if (UseProxy)
	gtk_entry_set_text(GTK_ENTRY(proxy_user), ProxyUser);

    proxy_password = gtk_entry_new_with_max_length(P_LENGTH-1);
    gtk_widget_set_name(proxy_password, "proxy_password");
    gtk_object_set_data(GTK_OBJECT(preferences), "proxy_password", proxy_password);
    gtk_widget_show(proxy_password);
    gtk_table_attach(GTK_TABLE(proxy_table), proxy_password, 1, 2, 3, 4,
		     (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 0, 0);
    gtk_entry_set_visibility(GTK_ENTRY(proxy_password), FALSE);
    if (UseProxy)
	gtk_entry_set_text(GTK_ENTRY(proxy_password), ProxyPass);

    config_label = gtk_frame_new(_("Configuration"));
    gtk_widget_show(config_label);
    gtk_box_pack_start(GTK_BOX(main_vbox), config_label, TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(config_label), 2);
    gtk_frame_set_label_align(GTK_FRAME(config_label), 0.02,0.5);

    config_vbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(config_vbox);
    gtk_container_add(GTK_CONTAINER(config_label), config_vbox);
    gtk_container_set_border_width(GTK_CONTAINER(config_vbox), 2);

    config_hbox1 = gtk_hbox_new(FALSE, 2);
    gtk_widget_show(config_hbox1);
    gtk_box_pack_start(GTK_BOX(config_vbox), config_hbox1, TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(config_hbox1), 2);

    debug_label = gtk_label_new(_("Debug Level"));
    gtk_widget_show(debug_label);
    gtk_box_pack_start(GTK_BOX(config_hbox1), debug_label, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(debug_label), GTK_JUSTIFY_RIGHT);

    debug_level_adj = gtk_adjustment_new(DebugLevel, 0, 15, 1, 5, 5);
    debug_level = gtk_spin_button_new(GTK_ADJUSTMENT(debug_level_adj), 1, 0);
    gtk_widget_set_name(debug_level, "debug_level");
    gtk_object_set_data(GTK_OBJECT(preferences), "debug_level", debug_level);
    gtk_widget_show(debug_level);
    gtk_box_pack_start(GTK_BOX(config_hbox1), debug_level, FALSE, FALSE, 0);
    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(debug_level), TRUE);

    tmout_label = gtk_label_new(_("Timeout"));
    gtk_widget_show(tmout_label);
    gtk_box_pack_start(GTK_BOX(config_hbox1), tmout_label, FALSE, FALSE, 0);
    gtk_label_set_justify(GTK_LABEL(tmout_label), GTK_JUSTIFY_RIGHT);

    timeout_level_adj = gtk_adjustment_new(TimeOut, 0, 1000, 10, 100, 100);
    timeout_level = gtk_spin_button_new(GTK_ADJUSTMENT(timeout_level_adj), 1, 0);
    gtk_widget_set_name(timeout_level, "timeout");
    gtk_object_set_data(GTK_OBJECT(preferences), "timeout", timeout_level);
    gtk_widget_show(timeout_level);
    gtk_box_pack_start(GTK_BOX(config_hbox1), timeout_level, FALSE, FALSE, 0);
    gtk_spin_button_set_numeric(GTK_SPIN_BUTTON(timeout_level), TRUE);

    config_hbox2 = gtk_hbox_new(FALSE, 2);
    gtk_widget_show(config_hbox2);
    gtk_box_pack_start(GTK_BOX(config_vbox), config_hbox2, FALSE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(config_hbox2), 2);

    auto_check = gtk_check_button_new_with_label(_("Auto check new version of scripts"));
    gtk_widget_set_name(auto_check, "auto_check");
    gtk_object_set_data(GTK_OBJECT(preferences), "auto_check", auto_check);
    gtk_widget_show(auto_check);
    gtk_box_pack_start(GTK_BOX(config_hbox2), auto_check, FALSE, FALSE, 0);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(auto_check), AutoCheck);

    keep_pass = gtk_check_button_new_with_label(_("Keep passwords"));
    gtk_widget_set_name(keep_pass, "keep_pass");
    gtk_object_set_data(GTK_OBJECT(preferences), "keep_pass", keep_pass);
    gtk_widget_show(keep_pass);
    gtk_box_pack_start(GTK_BOX(config_hbox2), keep_pass, FALSE, FALSE, 0);
    gtk_toggle_button_set_active(GTK_TOGGLE_BUTTON(keep_pass), KeepPass);

    dialog_action_area = GNOME_DIALOG(preferences)->action_area;
    gtk_widget_show(dialog_action_area);
    gtk_button_box_set_layout(GTK_BUTTON_BOX(dialog_action_area), GTK_BUTTONBOX_END);
    gtk_button_box_set_spacing(GTK_BUTTON_BOX(dialog_action_area), 8);

    gnome_dialog_append_button(GNOME_DIALOG(preferences), GNOME_STOCK_BUTTON_OK);
    ok_button = g_list_last(GNOME_DIALOG(preferences)->buttons)->data;
    gtk_widget_show(ok_button);
    GTK_WIDGET_SET_FLAGS(ok_button, GTK_CAN_DEFAULT);

    gnome_dialog_append_button(GNOME_DIALOG(preferences), GNOME_STOCK_BUTTON_APPLY);
    apply_button = g_list_last(GNOME_DIALOG(preferences)->buttons)->data;
    gtk_widget_show(apply_button);
    GTK_WIDGET_SET_FLAGS(apply_button, GTK_CAN_DEFAULT);

    gnome_dialog_append_button(GNOME_DIALOG(preferences), GNOME_STOCK_BUTTON_CANCEL);
    cancel_button = g_list_last(GNOME_DIALOG(preferences)->buttons)->data;
    gtk_widget_show(cancel_button);
    GTK_WIDGET_SET_FLAGS(cancel_button, GTK_CAN_DEFAULT);

    gtk_signal_connect(GTK_OBJECT(ok_button),
		      "pressed",
		      GTK_SIGNAL_FUNC(on_pref_ok_pressed),
		      (gpointer)1);
    gtk_signal_connect(GTK_OBJECT(apply_button),
		      "pressed",
		      GTK_SIGNAL_FUNC(on_pref_ok_pressed),
		      (gpointer)0);
    gtk_signal_connect(GTK_OBJECT(cancel_button),
		      "pressed",
		      GTK_SIGNAL_FUNC(on_pref_cancel_pressed),
		      NULL);

    return preferences;
}

/* gnome UI variables */
#include "updates_icon.xpm"
#include "home_icon.xpm"

static GnomeUIInfo file_menu_uiinfo[] = {
    GNOMEUIINFO_MENU_NEW_ITEM(N_("_Add provider File"), NULL, on_new_file_activate, NULL),
    {
     GNOME_APP_UI_ITEM, N_("_Go to SmsSend home page for more scripts"),
     NULL,
     on_goto_smssend_activate, NULL, NULL,
     GNOME_APP_PIXMAP_DATA, home_icon,
     0, 0, NULL},
    GNOMEUIINFO_SEPARATOR,
    GNOMEUIINFO_MENU_CLOSE_ITEM(on_close_activate, NULL),
    GNOMEUIINFO_MENU_EXIT_ITEM(on_exit_activate, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo settings_menu_uiinfo[] = {
    GNOMEUIINFO_MENU_PREFERENCES_ITEM(on_preferences_activate, NULL),
    {
     GNOME_APP_UI_ITEM, N_("Check for _updates"),
     NULL,
     on_check_updates_activate, NULL, NULL,
     GNOME_APP_PIXMAP_DATA, updates_icon,
     0, 0, NULL},
    GNOMEUIINFO_END
};

static GnomeUIInfo help_menu_uiinfo[] = {
    GNOMEUIINFO_MENU_ABOUT_ITEM(on_about_activate, NULL),
    GNOMEUIINFO_END
};

static GnomeUIInfo menubar_uiinfo[] = {
    GNOMEUIINFO_MENU_FILE_TREE(file_menu_uiinfo),
    GNOMEUIINFO_MENU_SETTINGS_TREE(settings_menu_uiinfo),
    GNOMEUIINFO_MENU_HELP_TREE(help_menu_uiinfo),
    GNOMEUIINFO_END
};

GtkWidget *create_gnome_smssend()
{
    /* create the application main window */
    GtkWidget *gnome_smssend;
    GtkWidget *dock;
    GtkWidget *appbar;

    tooltips = gtk_tooltips_new();

    gnome_smssend = gnome_app_new(PACKAGE, PACKAGE " - " VERSION);
    gtk_window_set_wmclass(GTK_WINDOW(gnome_smssend), "Smssend", "SmsSend");

    dock = GNOME_APP(gnome_smssend)->dock;
    gtk_widget_show(dock);

    gnome_app_create_menus(GNOME_APP(gnome_smssend), menubar_uiinfo);

    notebook = gtk_notebook_new();
    gtk_widget_show(notebook);
    gnome_app_set_contents(GNOME_APP(gnome_smssend), notebook);
    gtk_container_set_border_width(GTK_CONTAINER(notebook), 2);

    appbar = gnome_appbar_new(TRUE, TRUE, GNOME_PREFERENCES_NEVER);
    gtk_widget_set_name(appbar, "appbar");
    gtk_object_set_data(GTK_OBJECT(gnome_smssend), "appbar", appbar);
    gtk_widget_show(appbar);
    gnome_app_set_statusbar(GNOME_APP(gnome_smssend), appbar);

    return gnome_smssend;
}


PProvider_t create_provider(const char filename[])
{
    /* create a provider tab in the notebook */
    GtkWidget *provider_box;
    GtkWidget *param_frame;
    GtkWidget *table_params;
    GtkWidget *rightbox;
    GtkWidget *remaining;
    GtkWidget *send_button;
    GtkWidget *provider;
    GtkWidget *message=(GtkWidget *)NULL;

    gchar *label, *dot;
    PProvider_t Pv;

    int i, msgindex = -1;

    /* read provider script */
    Pv = CreateProviderFromFile(filename);

    if (Pv == NULL)
	return (NULL);

    /* generate provider name */
    label = g_strdup(g_basename(filename));
    dot   = (gchar *)g_extension_pointer (label);
    *--dot = '\000';
    provider = gtk_label_new(label);

    /* save filename and label in structure for DoSms() */
    Pv->Label    = label;
    Pv->Filename = g_strdup(filename);

    provider_box = gtk_hbox_new(FALSE, 2);
    gtk_widget_show(provider_box);
    gtk_container_add(GTK_CONTAINER(notebook), provider_box);
    gtk_container_set_border_width(GTK_CONTAINER(provider_box), 2);

    param_frame = gtk_frame_new(_("Parameters"));
    gtk_widget_show(param_frame);
    gtk_box_pack_start(GTK_BOX(provider_box), param_frame, TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(param_frame), 2);
    gtk_frame_set_label_align(GTK_FRAME(param_frame), 0.02, 0.5);

    table_params = gtk_table_new(Pv->NbParams, 2, FALSE);
    gtk_widget_show(table_params);
    gtk_container_add(GTK_CONTAINER(param_frame), table_params);
    gtk_container_set_border_width(GTK_CONTAINER(table_params), 2);
    gtk_table_set_row_spacings(GTK_TABLE(table_params), 2);
    gtk_table_set_col_spacings(GTK_TABLE(table_params), 2);

    /* create entries for the parameters */
    for (i = 0; i < Pv->NbParams; i++)
    {
	GtkWidget *param_label;
	GtkWidget *param_value;

	if (g_strncasecmp(Pv->Params[i].Name, "Message",9) == 0)
	{
	  /* message parameter is treated below */
	  msgindex = i;
	  break;
	}
	param_label = gtk_label_new(Pv->Params[i].Name);
	gtk_widget_show(param_label);
	gtk_table_attach(GTK_TABLE(table_params), param_label, 0, 1, i, i + 1,
			 (GtkAttachOptions) (0), (GtkAttachOptions) (0), 2, 2);
	gtk_label_set_justify(GTK_LABEL(param_label), GTK_JUSTIFY_LEFT);
	gtk_misc_set_padding(GTK_MISC(param_label), 2, 2);

	if (Pv->Params[i].Size)
	    param_value = gtk_entry_new_with_max_length(Pv->Params[i].Size);
	else
	    param_value = gtk_entry_new();
	gtk_widget_show(param_value);

	if (Pv->Params[i].Hidden)
	    gtk_entry_set_visibility(GTK_ENTRY(param_value), FALSE);

	gtk_table_attach(GTK_TABLE(table_params), param_value, 1, 2, i, i + 1,
			 (GtkAttachOptions) (GTK_EXPAND | GTK_FILL), (GtkAttachOptions) (0), 2, 2);

	Pv->Params[i].Widget = param_value;

	if (Pv->Params[i].Help)
	    gtk_tooltips_set_tip(tooltips, param_value, Pv->Params[i].Help, NULL);
	if (KeepPass && g_strncasecmp(Pv->Params[i].Name, "Password",10) == 0)
	{
	  gchar *config_file;
	  gchar *path;
	  gchar *password;
	  /* password is perhaps in preference file */
	  config_file=gnome_util_prepend_user_home(".smssend/smssendrc");
	  path = g_strdup_printf("=%s=/Passwords/%s",config_file,label);
	  password  = gnome_config_get_string_with_default(path,NULL);
	  if (password && strlen(password))
	  {
		crypt_password(password);
		Pv->Params[i].Value = password;
	  }
	  g_free(path);
	  g_free(config_file);	
	}
	if (Pv->Params[i].Value)
	    gtk_entry_set_text(GTK_ENTRY(param_value), Pv->Params[i].Value);
    }

    rightbox = gtk_vbox_new(FALSE, 2);
    gtk_widget_show(rightbox);
    gtk_box_pack_start(GTK_BOX(provider_box), rightbox, TRUE, TRUE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(rightbox), 2);

    /* message widget and remaining widget value */
    if (msgindex != -1)
    {
    	GtkWidget *remainbox;
	GtkWidget *remainlabl;
	char remain[4];

	message = gtk_text_new(NULL, NULL);
	gtk_widget_show(message);
	gtk_box_pack_start(GTK_BOX(rightbox), message, TRUE, FALSE, 0);
	gtk_text_set_editable(GTK_TEXT(message), TRUE);

	remainbox = gtk_hbox_new(TRUE, 2);
	gtk_widget_show(remainbox);
	gtk_box_pack_start(GTK_BOX(rightbox), remainbox, FALSE, TRUE, 0);
	gtk_container_set_border_width(GTK_CONTAINER(remainbox), 2);

	remainlabl = gtk_label_new(_("Remaining characters"));
	gtk_widget_show(remainlabl);
	gtk_box_pack_start(GTK_BOX(remainbox), remainlabl, FALSE, FALSE, 0);
	gtk_label_set_justify(GTK_LABEL(remainlabl), GTK_JUSTIFY_RIGHT);

	remaining = gtk_entry_new_with_max_length(3);
 	gtk_widget_show(remaining);
	gtk_box_pack_start(GTK_BOX(remainbox), remaining, FALSE, FALSE, 2);
	gtk_entry_set_editable(GTK_ENTRY(remaining), FALSE);
	gtk_tooltips_set_tip(tooltips, message, Pv->Params[msgindex].Help, NULL);
	g_snprintf(remain, 5, "%d", Pv->Params[msgindex].Size);
	gtk_entry_set_text(GTK_ENTRY(remaining), remain);
	Pv->Params[msgindex].Widget = message;
        gtk_signal_connect(GTK_OBJECT(message), "changed",
		       GTK_SIGNAL_FUNC(on_message_changed), (gpointer) Pv->Params[msgindex].Size);
	gtk_object_set_data(GTK_OBJECT(message), "remain", remaining);
    }

    /* send button */
    send_button = gnome_stock_button(GNOME_STOCK_BUTTON_APPLY);
    gtk_widget_show(send_button);
    gtk_box_pack_start(GTK_BOX(rightbox), send_button, FALSE, FALSE, 0);
    gtk_container_set_border_width(GTK_CONTAINER(send_button), 2);
    if (msgindex != -1)
    	gtk_signal_connect(GTK_OBJECT(send_button), "pressed", GTK_SIGNAL_FUNC(on_send_pressed), (gpointer) Pv);

    /* add to notebook with label */
    gtk_widget_show(provider);
    gtk_notebook_set_tab_label(GTK_NOTEBOOK(notebook), gtk_notebook_get_nth_page(GTK_NOTEBOOK(notebook), numprovider++), provider);

    /* cross save Pv and widget */
    Pv->Widget = message;
    gtk_object_set_data(GTK_OBJECT(provider_box), "provider", Pv);
    return (Pv);
}

static void scan_dir(gchar *dirname)
{
    DIR           *dir;
    struct dirent *dirent;
    gchar         *ext;
    PProvider_t   Pv = NULL;

    dir = opendir(dirname);
    if (!dir)
	return;
    while ((dirent = readdir(dir)))
    {
	/* Search for SMSSEND extension */
	ext = (gchar *)g_extension_pointer(dirent->d_name);
	if (g_strncasecmp(g_extension_pointer(dirent->d_name),SMSSEND_SUF,3) == 0)
	{
    	    Pv = create_provider(g_strconcat(dirname,"/",dirent->d_name,NULL));
	    if (!Pv)
		g_printerr(_("Error for '%s'!\n"),g_strconcat(dirname,"/",dirent->d_name,NULL));
	}
    }        
    closedir(dir);
}

void create_notebook()
{
    gchar *dirname;

    /* Create all notebook's tabs */
    numprovider = 0;

    /* First try to load local .sms script */
    scan_dir(g_get_current_dir());
    /* Then try to load user script */
    scan_dir(gnome_util_prepend_user_home(".smssend/"));
    /* Then try to load global shared scripts */
    scan_dir(SMSSEND_SHAREPATH);
    /* Use an environment variable */
    if ( (dirname = getenv("SMSSEND_SCRIPTPATH")) != NULL)    
    {
        scan_dir(dirname);
    }
    
    if (numprovider == 0)
    {
	g_printerr(_("No provider file found!\n"));
    	gtk_exit(-1);
    }
}
